import os

from pydantic import BaseSettings


class ProductionConfig(BaseSettings):
    app_name: str = "Gigalogy Platform Monitor API"
    app_description: str = "Monitors all the containers running in Gigalogy platform and send slack notification when something goes wrong"
    app_version: str = "v1"
    app_logo_url: str = "https://www.gigalogy.com/img/favicon.png"
    env_name: str = "PROD"
    allowed_hosts: str = os.environ["ALLOWED_HOSTS"]
    slack_webhook_url: str = os.environ["SLACK_WEBHOOK_URL"]


class StagingConfig(BaseSettings):
    app_name: str = "Gigalogy Platform Monitor API"
    app_description: str = "Monitors all the containers running in Gigalogy platform and send slack notification when something goes wrong"
    app_version: str = "v1"
    app_logo_url: str = "https://www.gigalogy.com/img/favicon.png"
    env_name: str = "STG"
    allowed_hosts: str = os.environ["ALLOWED_HOSTS"]
    slack_webhook_url: str = os.environ["SLACK_WEBHOOK_URL"]


if os.environ["APP_ENVIRONMENT"] == "PROD":
    config = ProductionConfig()
elif os.environ["APP_ENVIRONMENT"] == "STG":
    config = StagingConfig()


import docker
from fastapi import FastAPI
from starlette.middleware.cors import CORSMiddleware
from starlette.middleware.trustedhost import TrustedHostMiddleware

from glitor.config import config

client = docker.from_env()


def get_app():
    app = FastAPI(
        title=config.app_name,
        description=config.app_description,
        version=config.app_version,
    )

    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    app.add_middleware(TrustedHostMiddleware, allowed_hosts=config.allowed_hosts)

    return app





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['glitor']

package_data = \
{'': ['*']}

install_requires = \
['docker>=5.0.3,<6.0.0',
 'fastapi-utils>=0.2.1,<0.3.0',
 'fastapi>=0.71.0,<0.72.0',
 'gunicorn>=20.1.0,<21.0.0',
 'uvicorn>=0.16.0,<0.17.0']

setup_kwargs = {
    'name': 'glitor',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'Md Moin Uddin',
    'author_email': 'moensam@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import json

import requests
from fastapi_utils.tasks import repeat_every

from glitor.asgi import app
from glitor.config import config
from glitor.main import client


def resource_usage(response):
    try:
        precpu_total = response["precpu_stats"]["cpu_usage"]["total_usage"]
    except:
        precpu_total = 0

    try:
        precpu_system = response["precpu_stats"]["system_cpu_usage"]
    except:
        precpu_system = 0

    try:
        system_cpu_usage = response["cpu_stats"]["system_cpu_usage"]
    except:
        system_cpu_usage = 0

    try:
        cpu_total_usage = response["cpu_stats"]["cpu_usage"]["total_usage"]
    except:
        cpu_total_usage = 0

    try:
        online_cpus = response["cpu_stats"]["online_cpus"]
    except:
        online_cpus = 0

    try:
        memory_stats_usage = response["memory_stats"]["usage"]
    except:
        memory_stats_usage = 0

    try:
        memory_stats_cache = response["memory_stats"]["stats"]["cache"]
    except:
        memory_stats_cache = 0

    try:
        memory_stats_limit = response["memory_stats"]["limit"]
    except:
        memory_stats_limit = 0

    cpu_delta = cpu_total_usage - precpu_total
    system_delta = system_cpu_usage - precpu_system
    cpu_usage = cpu_delta / system_delta * online_cpus * 100 if system_delta != 0 else 0

    memory_usage = memory_stats_usage - memory_stats_cache
    memory_usage = (
        memory_usage / memory_stats_limit * 100 if memory_stats_limit != 0 else 0
    )

    return cpu_usage, memory_usage


def send_message_to_slack(container_name, container_status, cpu_usage, memory_usage):
    hook = config.slack_webhook_url
    headers = {"content-type": "application/json"}

    COLOR = "#FF0000"
    TITLE = f"[Usage Alert] | {container_name} | {container_status}"
    MESSAGE = f"CPU Usage: {cpu_usage} % | Total Memory Usage: {memory_usage} %"
    payload = {
        "attachments": [
            {
                "fallback": "",
                "pretext": "",
                "color": COLOR,
                "fields": [{"title": TITLE, "value": MESSAGE, "short": False}],
            }
        ]
    }
    r = requests.post(hook, data=json.dumps(payload), headers=headers)
    print("Response: " + str(r.status_code) + "," + str(r.reason))


@app.on_event("startup")
@repeat_every(seconds=60, raise_exceptions=True)
def resource_usage_alert():
    total_memory_usage = 0
    for container in client.containers.list(all=True):
        stats_json = container.stats(stream=False)
        res_usage = resource_usage(stats_json)
        cpu_usage = round(res_usage[0], 2)
        memory_usage = round(res_usage[1], 2)
        total_memory_usage = total_memory_usage + memory_usage
        if cpu_usage > 90 or total_memory_usage > 75 or container.status != "running":
            send_message_to_slack(
                container.name,
                container.status,
                cpu_usage,
                round(total_memory_usage, 2),
            )
